function plot_toy_problem(Xl,Yl,Xu,Xt,Yt)
% PLOT_TOY_PROBLEM(Xl,Yl,Xu,Xt,Yt)
% plot a 2D toy problem. 
% Typically, Xl, Yl, Xu, Xt are returned by
% GENERATE_TWO_GAUSSIANS or GENERATE_TWO_MOONS
% Yt, if not empty  is the predicted (real) output of the test points.
 
 xmin=min(Xt(:,1)); 
 xmax=max(Xt(:,1)); 
 ymin=min(Xt(:,2)); 
 ymax=max(Xt(:,2));
 cla;
 set(gca,'XLim',[xmin xmax],'YLim',[ymin ymax]);
 hold on
 
 if ~isempty(Yt)
   [x,y] = meshgrid(xmin:(xmax-xmin)/50:xmax,ymin:(ymax-ymin)/50:ymax);
   Xt2=[reshape(x,prod(size(x)),1) reshape(y,prod(size(y)),1)];
   if ~isequal(Xt,Xt2)
     error('Incorrect Xt');
   end;
   
   Yt=reshape(Yt,size(x));
   
   sp = pcolor(x,y,Yt);
   shading interp
   set(sp,'LineStyle','none');
   axis off
   colormap((1+gray)/2);
   colormap(cool)
   [dummy c1] = contour(x,y,Yt,[0 0],'w');
   set(c1,'LineWidth',2);
   [dummy c2] = contour(x,y,Yt,[-1 1],'w');
 end;

 for i=1:size(Xu,1)
   plot(Xu(i,1),Xu(i,2),'k.','Markersize',7);
 end;
 
 for i=1:length(Yl)
   if Yl(i)==1
     plot(Xl(i,1),Xl(i,2),'b^','MarkerSize',15,'Linewidth',3,...
          'Color',[1 1 0]);
   else
     plot(Xl(i,1),Xl(i,2),'y+','MarkerSize',15,'Linewidth',3);
   end;
 end;

 hold off;
